@@ -5,9 +5,10 @@ from .config import Config
 from .network import Network
 from . import utils
 
+import os
 import logging
 from typing import Optional, Union
-from collections.abc import Iterable
+from collections.abc import Iterable, Mapping
 
 import matplotlib.pyplot as plt
 from matplotlib import colors as mpl_colors
@@ -17,6 +18,7 @@ from matplotlib.patches import Patch
 import distinctipy
 import tomli
 import tomli_w
+import pandas as pd
 
 class Colors():
     
@@ -50,6 +52,8 @@ class Colors():
             return cls(**tomli.load(f))
 
     def to_toml(self, toml_file: str):
+        if os.path.exists(toml_file):
+            logging.warning(f"Overwriting {toml_file}")
         with open(toml_file, "wb") as f:
             tomli_w.dump(self.__dict__, f)
 
@@ -72,6 +76,16 @@ class Colors():
         colors.add_missing_metadata_colors(datasets=integration, pastel_factor=pastel_factor, colorblind_type=colorblind_type)
         return colors
     
+    @classmethod
+    def from_named_datasets(cls, 
+                     datasets: Mapping[str, Dataset],
+                     pastel_factor=0.3,
+                     colorblind_type=None):
+        colors = cls()
+        colors.add_missing_dataset_colors(datasets=datasets.keys(), pastel_factor=pastel_factor, colorblind_type=colorblind_type)
+        colors.add_missing_metadata_colors(datasets=datasets, pastel_factor=pastel_factor, colorblind_type=colorblind_type)
+        return colors
+    
     @classmethod
     def from_network(cls, 
                  network: Network,
@@ -140,7 +154,7 @@ class Colors():
         invalid_colors = [name for name, color in self.community_colors.items()
                           if not mpl_colors.is_color_like(color)]
         if invalid_colors:
-            raise ValueError(f"Communitieswere given these invalid colors: {invalid_colors}. Please use valid matplotlib colors in named, hex, or RGB formats.")
+            raise ValueError(f"Communities were given these invalid colors: {invalid_colors}. Please use valid matplotlib colors in named, hex, or RGB formats.")
 
         # fill in missing values with random colors distinct from existing colors
         uncolored_communities = set(communities) - self.community_colors.keys()
@@ -199,7 +213,7 @@ class Colors():
             return fig
         
     def add_missing_metadata_colors(self,
-                                    datasets: Union[Dataset, Integration],
+                                    datasets: Union[Dataset, Integration, Mapping[str, Dataset]],
                                     pastel_factor=0.3,
                                     colorblind_type=None):
         """
@@ -209,6 +223,8 @@ class Colors():
         # get categorical data for which colors should match
         if isinstance(datasets, Dataset):
             metadata_df = datasets.get_metadata_df(include_numerical=False)
+        elif isinstance(datasets, Mapping):
+            metadata_df = pd.concat({dataset_name: dataset.get_metadata_df(include_numerical=False) for dataset_name, dataset in datasets.items()})
         elif isinstance(datasets, Integration):
             metadata_df = datasets.get_metadata_df(include_numerical=False, prepend_dataset_column=False)
         else:
@@ -259,19 +275,22 @@ class Colors():
         return layer_colors
 
     def plot_metadata_colors_legend(self,
-                                    layer: str = None,
+                                    categories: str = None,
                                     ax: Axes = None,
                                     char_per_line: int = 20,
+                                    subset = None,
                                     figsize: Iterable = None) -> Optional[Figure]:
         
-        if layer is not None and isinstance(ax, Axes):
-            color_def = self.metadata_colors[layer]
+        if categories is not None and isinstance(ax, Axes):
+            color_def = self.metadata_colors[categories]
+            if subset is not None:
+                color_def = {k:v for k, v in color_def.items() if k in subset}
             legend_elements = [Patch(label=utils.newline_wrap(cat, char_per_line), facecolor=color, edgecolor=None) for cat, color in color_def.items()]
             ax.legend(handles=legend_elements, loc='upper center')
-            ax.set_title(layer)
+            ax.set_title(categories)
             ax.set_axis_off()
                 
-        elif layer is None and ax is None:
+        elif categories is None and ax is None:
         
             categorical_columns = [track for track, color_def in self.metadata_colors.items() if isinstance(color_def, dict)]
             categorical_groups = [group for group, group_attr in self.metadata_colors_group.items() if isinstance(group_attr["colors"], dict)]
@@ -283,11 +302,11 @@ class Colors():
             if figsize is None:
                 figsize = [2*n_columns, 0.3 * max(legend_lengths)]
             fig, axes = plt.subplots(1, n_columns, figsize=figsize, squeeze=False, layout="tight")
-            for ax_layer, layer in zip(axes[0], categorical_columns):
-                color_def = self.metadata_colors[layer]
+            for ax_layer, categories in zip(axes[0], categorical_columns):
+                color_def = self.metadata_colors[categories]
                 legend_elements = [Patch(label=utils.newline_wrap(cat, char_per_line), facecolor=color, edgecolor=None) for cat, color in color_def.items()]
                 ax_layer.legend(handles=legend_elements, loc='upper center')
-                ax_layer.set_title(layer)
+                ax_layer.set_title(categories)
                 ax_layer.set_axis_off()
             for ax_id, group in enumerate(categorical_groups, len(categorical_columns)):
                 ax_layer = axes[0][ax_id]
@@ -301,7 +320,7 @@ class Colors():
             axes[0][-1].legend(handles=[Patch(label="Missing Data", facecolor=self.missing_data_color, edgecolor=None)], loc='upper left')
             axes[0][-1].set_axis_off()
         else:
-            raise ValueError("Parameters `layer` and `ax` must both be specified")
+            raise ValueError("Parameters `categories` and `ax` must both be specified")
         
         if ax is None:
             return fig
\ No newline at end of file
