@@ -121,12 +121,12 @@ class Network():
         node_table[("network", "")] = pd.Series([node_to_program(node)[0] for node in self.program_graph.nodes]).value_counts()
         return node_table
 
-    def get_community_usage(self,
+    def get_median_community_usage(self,
                             subset_datasets: Optional[Union[str, Iterable[str]]] = None,
                             normalize: bool = True,
                             discretize: bool = False):
         """
-        Get median usage of each community of programs for each samples.  # TODO: migrate to representative programs
+        Get median usage of each community of programs for each sample.
 
         :param subset_datasets: dataset name or iterable of dataset names to subset the results, defaults to None
         :type subset_datasets: str or Iterable[str], optional
@@ -138,7 +138,7 @@ class Network():
         :rtype: pd.DataFrame
         """
         usage = self.integration.get_usages(normalize=normalize)
-        ic_usage = []
+        median_usage = []
         
         if subset_datasets is None:
             subset_datasets = self.integration.datasets
@@ -158,13 +158,13 @@ class Network():
                 data.append(program_comm.median(axis=1).rename((community, dataset_name)))
             data = pd.concat(data, axis=1).droplevel(axis=1, level=1).dropna(how="all")
             data.columns.rename("Community", inplace=True)
-            ic_usage.append(data.sort_index(axis=0))
-        ic_usage = pd.concat(ic_usage)
+            median_usage.append(data.sort_index(axis=0))
+        median_usage = pd.concat(median_usage)
         if normalize:
-            ic_usage = ic_usage.div(ic_usage.sum(axis=1), axis=0)
+            median_usage = median_usage.div(median_usage.sum(axis=1), axis=0)
         if discretize:
-            ic_usage = ic_usage.eq(ic_usage.max(axis=1), axis=0).astype(int)
-        return ic_usage
+            median_usage = median_usage.eq(median_usage.max(axis=1), axis=0).astype(int)
+        return median_usage
     
     def get_sample_entropy(self, subset_datasets: Optional[Union[str, Iterable]] = None):
         """Get shannon diversity of Community Usage for each sample.
@@ -174,7 +174,7 @@ class Network():
         :return: Shannon Entropy for each dataset and sample
         :rtype: pd.Series
         """
-        ic_usage = self.get_community_usage(subset_datasets = subset_datasets)
+        ic_usage = self.get_median_community_usage(subset_datasets = subset_datasets)
         diversity = ic_usage.apply(lambda x: entropy(x.dropna()), axis=1)
         return diversity
     
@@ -201,7 +201,7 @@ class Network():
                                                               subset_datasets=subset_datasets,
                                                               truncate_negative=False,
                                                               subset_categories=subset_categories)
-        mapper = {tuple([program.split("|")[0], int(program.split("|")[1]), int(program.split("|")[2])]): comm for program, comm in self.program_communities.items()}
+        mapper = {tuple([program.split("|")[0], int(program.split("|")[1]), int(program.split("|")[2])]): comm for program, comm in self.node_to_community.items()}
         df.columns = df.columns.map(mapper)
         df = df.T.groupby(level=0).mean().T
         df = df.reindex(self.ordered_community_names, axis=1)
@@ -227,7 +227,7 @@ class Network():
         """
 
         ser = self.integration.get_metadata_correlation(layer=layer, subset_datasets=subset_datasets, method=method)
-        mapper = {tuple([program.split("|")[0], int(program.split("|")[1]), int(program.split("|")[2])]): comm for program, comm in self.program_communities.items()}
+        mapper = {tuple([program.split("|")[0], int(program.split("|")[1]), int(program.split("|")[2])]): comm for program, comm in self.node_to_community.items()}
         ser.index = ser.index.map(mapper)
         ser = ser.groupby(level=0).mean()
         ser = ser.reindex(self.ordered_community_names)
@@ -331,7 +331,6 @@ class Network():
         communities = {str(k): v for k, v in communities.items()}
 
         self.communities = communities
-        self.program_communities = {program: community for community, programs in communities.items() for program in programs}
 
         # also create a community network with default parameters, so that self.comm_graph is available if needed. However, self.create_community_network can be called again
         self.create_community_network()
@@ -345,7 +344,6 @@ class Network():
         communities = {str(k): v for k, v in communities.items()}
         
         self.communities = communities
-        self.program_communities = {program: community for community, programs in communities.items() for program in programs}
 
         # also create a community network with default parameters, so that self.comm_graph is available if needed. However, self.create_community_network can be called again
         self.create_community_network()
@@ -392,7 +390,6 @@ class Network():
             pruned = renumbered
         
         self.communities = pruned
-        self.program_communities = {program: community for community, programs in self.communities.items() for program in programs}
         self.create_community_network()
         self.program_graph = self.program_graph.subgraph([node for nodes in self.communities.values() for node in nodes])  # removes nodes not in a pruned community
 
@@ -407,7 +404,12 @@ class Network():
         community_names = sorted(self.communities.keys(), key = lambda cstr: [int(lvl) for lvl in cstr.split(".")])
         return community_names
     
-    
+    @property
+    def node_to_community(self) -> Dict[str, str]:
+        dict_of_programs = {program: community for community, programs in self.communities.items() for program in programs}
+        return dict_of_programs
+
+
     def get_vectorized_community_sort_key(self, community_names: pd.Index) -> pd.Index:
         """Return a vector of sort_indicesGet community names, ordered numerically after separating clusters and subclusters. For example, this algorithm can properly sort communities labelled 1.1, 1.2, 1.3, 2.1, 2.2, 2.10, 2.15.
 
@@ -430,8 +432,8 @@ class Network():
         edge_attr = {}
         for edge in self.program_graph.edges:
             weight = base_weight
-            if edge[0] in self.program_communities and edge[1] in self.program_communities:  # nodes might not have communities due to pruning, and so will be treated as if they are in different communities for layout purposes
-                if self.program_communities[edge[0]] == self.program_communities[edge[1]]:
+            if edge[0] in self.node_to_community and edge[1] in self.node_to_community:  # nodes might not have communities due to pruning, and so will be treated as if they are in different communities for layout purposes
+                if self.node_to_community[edge[0]] == self.node_to_community[edge[1]]:
                     weight *= shared_community_weight
             if edge[0].split("|")[0] == edge[1].split("|")[0]:
                 weight *= shared_dataset_weight
@@ -662,7 +664,7 @@ class Network():
             
 
         elif correlation_axis == "usage":
-            cu = self.get_community_usage()
+            cu = self.get_median_community_usage()
             usages = self.integration.get_usages(normalize=True)
             for community, nodes in self.communities.items():
                 for dataset_name in self.integration.datasets:
@@ -687,7 +689,7 @@ class Network():
         :return: Communities, indexed by the lowest rank program(s) for each dataset
         :rtype: pd.Series
         """
-        programs = pd.Series(self.program_communities, name="Community")
+        programs = pd.Series(self.node_to_community, name="Community")
         programs.index = pd.MultiIndex.from_tuples((node_to_program(node) for node in programs.index), names=["dataset", "k", "program"])
         programs = programs.sort_index()
         if isinstance(k, int):
@@ -712,8 +714,25 @@ class Network():
         """
         rep_programs_ids = self.get_representative_program_ids(correlation_axis=correlation_axis)
         rep_programs = self.integration.get_programs()[rep_programs_ids.index]
-        rep_programs.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["Community", "dataset", "k", "Program"])
-        return rep_programs  # TODO: Check return types
+        rep_programs.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["community", "dataset", "k", "program"])
+        return rep_programs
+
+
+    def get_representative_program_usage(self,
+                         correlation_axis: Literal["programs", "usage"] = "programs"
+                         ) -> pd.DataFrame:
+        """Select programs based on correlation with the median of all programs in each community
+        
+        :param correlation_axis: axis on which to compute correlations between programs whether correlating the programs or the usages, defaults to "programs"
+        :type correlation_axis: int or dict
+        :return: Features × programs matrix
+        :rtype: pd.DataFrame
+        """
+        rep_programs_ids = self.get_representative_program_ids(correlation_axis=correlation_axis)
+        rep_programs = self.integration.get_usages()[rep_programs_ids.index]
+        rep_programs.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["community", "dataset", "k", "program"])
+        return rep_programs
+
 
     def get_lowest_rank_programs(self,
                                 min_k: Optional[Union[int, Dict[str, int]]] = None,
@@ -726,7 +745,7 @@ class Network():
         :return: Communities, indexed by the lowest rank program(s) for each dataset
         :rtype: pd.Series
         """
-        programs = pd.Series(self.program_communities, name="Community")
+        programs = pd.Series(self.node_to_community, name="Community")
         programs.index = pd.MultiIndex.from_tuples((node_to_program(node) for node in programs.index), names=["dataset", "k", "program"])
         programs = programs.sort_index()
         if isinstance(min_k, int):
@@ -829,7 +848,7 @@ class Network():
         :return: Communities-Datasets × variables consensus matrix
         :rtype: pd.DataFrame
         """
-        program_to_community = {node_to_program(node): community for node, community in self.program_communities.items()}
+        program_to_community = {node_to_program(node): community for node, community in self.node_to_community.items()}
         programs = self.integration.get_programs()
         programs = programs.loc[:, programs.columns.get_level_values(1).astype(int) >= min_k]
         programs.columns = pd.MultiIndex.from_arrays([programs.columns.map(program_to_community),
@@ -850,7 +869,7 @@ class Network():
     def transfer_labels(self,
                           source: Optional[Union[str, Collection[str]]] = None,
                           dest: Optional[Union[str, Collection[str]]] = None,
-                          layer: Optional[Union[str, Collection[str]]] = None,
+                          categories: Optional[Union[str, Collection[str]]] = None,
                           subset_categories: Collection[str] = None,
                           simplify: bool = True
                           ) -> pd.DataFrame:
@@ -860,8 +879,8 @@ class Network():
         :type source: Union[str, Collection[str]], optional
         :param dest: Target dataset(s) for label transfer, defaults to None
         :type dest: Union[str, Collection[str]], optional
-        :param layer: name of categorical data layer(s) from source dataset, defaults to None
-        :type layer: Union[str, Collection[str]], optional
+        :param categories: name of categorical metadata field(s) from source dataset, defaults to None
+        :type categories: Union[str, Collection[str]], optional
         :param subset_categories: a subset of categories for calculating overrepresentation, defaults to None
         :type subset_categories: Collection[str], optional
         :param simplify: Simplify multi-index results when only one source, dest, or layer are specified, defaults to True
@@ -881,13 +900,13 @@ class Network():
         else:
             raise ValueError
         
-        if layer is None:
+        if categories is None:
             # get all layers for all selected source datasets
             layers = self.integration.get_metadata_df(include_numerical=False, subset_datasets=sources).columns
-        elif isinstance(layer, str):
-            layers = [layer]
-        elif isinstance(layer, Collection):
-            layers = layer
+        elif isinstance(categories, str):
+            layers = [categories]
+        elif isinstance(categories, Collection):
+            layers = categories
         else:
             raise ValueError
         
@@ -931,7 +950,7 @@ class Network():
         agg = pd.concat(agg)
 
         if simplify:
-            if isinstance(layer, str):
+            if isinstance(categories, str):
                 agg = agg.droplevel(axis=0, level="layer")
             if isinstance(source, str):
                 agg = agg.droplevel(axis=0, level="source_dataset")
@@ -961,7 +980,7 @@ def compare_community_jaccard_similarity(name1: str, network1: Network, name2: s
     net1_datasets = set(network1.integration.datasets) - shared_datasets
     net2_datasets = set(network2.integration.datasets) - shared_datasets
 
-    jaccard = pd.DataFrame(np.NaN,
+    jaccard = pd.DataFrame(np.nan,
                            index=pd.Index(network1.ordered_community_names, name=name1),
                            columns=pd.Index(network2.ordered_community_names, name=name2))
     for net1_comm, net1_nodes in network1.communities.items():
